{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "d8fbc8e2",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:15.349957Z",
     "iopub.status.busy": "2024-08-12T10:00:15.349796Z",
     "iopub.status.idle": "2024-08-12T10:00:15.686916Z",
     "shell.execute_reply": "2024-08-12T10:00:15.686351Z"
    }
   },
   "outputs": [],
   "source": [
    "import sympy as sm\n",
    "import sympy.physics.mechanics as me\n",
    "me.init_vprinting(use_latex='mathjax')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "e633f9c1",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:15.689231Z",
     "iopub.status.busy": "2024-08-12T10:00:15.688839Z",
     "iopub.status.idle": "2024-08-12T10:00:15.692441Z",
     "shell.execute_reply": "2024-08-12T10:00:15.691988Z"
    }
   },
   "outputs": [],
   "source": [
    "class ReferenceFrame(me.ReferenceFrame):\n",
    "\n",
    "    def __init__(self, *args, **kwargs):\n",
    "\n",
    "        kwargs.pop('latexs', None)\n",
    "\n",
    "        lab = args[0].lower()\n",
    "        tex = r'\\hat{{{}}}_{}'\n",
    "\n",
    "        super(ReferenceFrame, self).__init__(*args,\n",
    "                                             latexs=(tex.format(lab, 'x'),\n",
    "                                                     tex.format(lab, 'y'),\n",
    "                                                     tex.format(lab, 'z')),\n",
    "                                             **kwargs)\n",
    "me.ReferenceFrame = ReferenceFrame"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "c019e0b2",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:15.694215Z",
     "iopub.status.busy": "2024-08-12T10:00:15.693923Z",
     "iopub.status.idle": "2024-08-12T10:00:15.707720Z",
     "shell.execute_reply": "2024-08-12T10:00:15.707350Z"
    }
   },
   "outputs": [],
   "source": [
    "q1, q2, q3 = me.dynamicsymbols('q1, q2, q3')\n",
    "la, lb, lc, ln = sm.symbols('l_a, l_b, l_c, l_n')\n",
    "\n",
    "N = me.ReferenceFrame('N')\n",
    "A = me.ReferenceFrame('A')\n",
    "B = me.ReferenceFrame('B')\n",
    "C = me.ReferenceFrame('C')\n",
    "\n",
    "A.orient_axis(N, q1, N.z)\n",
    "B.orient_axis(A, q2, A.z)\n",
    "C.orient_axis(B, q3, B.z)\n",
    "\n",
    "P1 = me.Point('P1')\n",
    "P2 = me.Point('P2')\n",
    "P3 = me.Point('P3')\n",
    "P4 = me.Point('P4')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "ca745fee",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:15.709436Z",
     "iopub.status.busy": "2024-08-12T10:00:15.709292Z",
     "iopub.status.idle": "2024-08-12T10:00:15.715290Z",
     "shell.execute_reply": "2024-08-12T10:00:15.714772Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{c}\\hat{c}_x + l_{b}\\hat{b}_x + l_{a}\\hat{a}_x$"
      ],
      "text/plain": [
       "l_c c_x + l_b b_x + lₐ a_x"
      ]
     },
     "execution_count": 4,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "P2.set_pos(P1, la*A.x)\n",
    "P3.set_pos(P2, lb*B.x)\n",
    "P4.set_pos(P3, lc*C.x)\n",
    "\n",
    "P4.pos_from(P1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "id": "8e3510da",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:15.717199Z",
     "iopub.status.busy": "2024-08-12T10:00:15.716838Z",
     "iopub.status.idle": "2024-08-12T10:00:15.719285Z",
     "shell.execute_reply": "2024-08-12T10:00:15.718880Z"
    }
   },
   "outputs": [],
   "source": [
    "r_P1_P4 = ln*N.x"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "cdf227be",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:15.720912Z",
     "iopub.status.busy": "2024-08-12T10:00:15.720774Z",
     "iopub.status.idle": "2024-08-12T10:00:15.725205Z",
     "shell.execute_reply": "2024-08-12T10:00:15.724823Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{c}\\hat{c}_x + l_{b}\\hat{b}_x + l_{a}\\hat{a}_x -  l_{n}\\hat{n}_x$"
      ],
      "text/plain": [
       "l_c c_x + l_b b_x + lₐ a_x + -lₙ n_x"
      ]
     },
     "execution_count": 6,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "loop = P4.pos_from(P1) - r_P1_P4\n",
    "loop"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "a81b2938",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:15.726777Z",
     "iopub.status.busy": "2024-08-12T10:00:15.726639Z",
     "iopub.status.idle": "2024-08-12T10:00:16.034423Z",
     "shell.execute_reply": "2024-08-12T10:00:16.033885Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{a} \\cos{\\left(q_{1} \\right)} + l_{b} \\cos{\\left(q_{1} + q_{2} \\right)} + l_{c} \\cos{\\left(q_{1} + q_{2} + q_{3} \\right)} - l_{n}$"
      ],
      "text/plain": [
       "lₐ⋅cos(q₁) + l_b⋅cos(q₁ + q₂) + l_c⋅cos(q₁ + q₂ + q₃) - lₙ"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "fhx = sm.trigsimp(loop.dot(N.x))\n",
    "fhx"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "7e55016f",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.036279Z",
     "iopub.status.busy": "2024-08-12T10:00:16.036094Z",
     "iopub.status.idle": "2024-08-12T10:00:16.279294Z",
     "shell.execute_reply": "2024-08-12T10:00:16.278818Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{a} \\sin{\\left(q_{1} \\right)} + l_{b} \\sin{\\left(q_{1} + q_{2} \\right)} + l_{c} \\sin{\\left(q_{1} + q_{2} + q_{3} \\right)}$"
      ],
      "text/plain": [
       "lₐ⋅sin(q₁) + l_b⋅sin(q₁ + q₂) + l_c⋅sin(q₁ + q₂ + q₃)"
      ]
     },
     "execution_count": 8,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "fhy = sm.trigsimp(loop.dot(N.y))\n",
    "fhy"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "id": "26a31b8a",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.281393Z",
     "iopub.status.busy": "2024-08-12T10:00:16.281043Z",
     "iopub.status.idle": "2024-08-12T10:00:16.288875Z",
     "shell.execute_reply": "2024-08-12T10:00:16.288504Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}l_{a} \\cos{\\left(q_{1} \\right)} + l_{b} \\cos{\\left(q_{1} + q_{2} \\right)} + l_{c} \\cos{\\left(q_{1} + q_{2} + q_{3} \\right)} - l_{n}\\\\l_{a} \\sin{\\left(q_{1} \\right)} + l_{b} \\sin{\\left(q_{1} + q_{2} \\right)} + l_{c} \\sin{\\left(q_{1} + q_{2} + q_{3} \\right)}\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡lₐ⋅cos(q₁) + l_b⋅cos(q₁ + q₂) + l_c⋅cos(q₁ + q₂ + q₃) - lₙ⎤\n",
       "⎢                                                          ⎥\n",
       "⎣  lₐ⋅sin(q₁) + l_b⋅sin(q₁ + q₂) + l_c⋅sin(q₁ + q₂ + q₃)   ⎦"
      ]
     },
     "execution_count": 9,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "fh = sm.Matrix([fhx, fhy])\n",
    "fh"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "d9166c9c",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.290546Z",
     "iopub.status.busy": "2024-08-12T10:00:16.290403Z",
     "iopub.status.idle": "2024-08-12T10:00:16.720369Z",
     "shell.execute_reply": "2024-08-12T10:00:16.719879Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}2 a \\cos{\\left(q_{1} + q_{2} \\right)} + b \\cos{\\left(q_{1} + q_{2} + q_{3} \\right)} + b \\cos{\\left(q_{1} \\right)} - \\frac{39 b}{20}\\\\2 a \\sin{\\left(q_{1} + q_{2} \\right)} + 2 a + b \\sin{\\left(q_{1} + q_{2} + q_{3} \\right)} + b \\sin{\\left(q_{1} \\right)}\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡                                                     39⋅b⎤\n",
       "⎢2⋅a⋅cos(q₁ + q₂) + b⋅cos(q₁ + q₂ + q₃) + b⋅cos(q₁) - ────⎥\n",
       "⎢                                                      20 ⎥\n",
       "⎢                                                         ⎥\n",
       "⎣2⋅a⋅sin(q₁ + q₂) + 2⋅a + b⋅sin(q₁ + q₂ + q₃) + b⋅sin(q₁) ⎦"
      ]
     },
     "execution_count": 10,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "q1, q2, q3 = me.dynamicsymbols('q1, q2, q3')\n",
    "a, b = sm.symbols('a, b')\n",
    "\n",
    "N = me.ReferenceFrame('N')\n",
    "A = me.ReferenceFrame('A')\n",
    "B = me.ReferenceFrame('B')\n",
    "C = me.ReferenceFrame('C')\n",
    "\n",
    "A.orient_axis(N, q1, N.z)\n",
    "B.orient_axis(A, q2, A.z)\n",
    "C.orient_axis(B, q3, B.z)\n",
    "\n",
    "P1 = me.Point('P1')\n",
    "P2 = me.Point('P2')\n",
    "P3 = me.Point('P3')\n",
    "P4 = me.Point('P4')\n",
    "\n",
    "P2.set_pos(P1, b*A.x)\n",
    "P3.set_pos(P2, 2*a*B.x)\n",
    "P4.set_pos(P3, b*C.x)\n",
    "\n",
    "P4.pos_from(P1)\n",
    "\n",
    "r_P1_P4 = (2 - sm.S(1)/20)*b*N.x - 2*a*N.y\n",
    "\n",
    "loop = P4.pos_from(P1) - r_P1_P4\n",
    "\n",
    "fh_watts = sm.trigsimp(sm.Matrix([loop.dot(N.x), loop.dot(N.y)]))\n",
    "fh_watts"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "id": "39a70215",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.722217Z",
     "iopub.status.busy": "2024-08-12T10:00:16.722040Z",
     "iopub.status.idle": "2024-08-12T10:00:16.726379Z",
     "shell.execute_reply": "2024-08-12T10:00:16.725996Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left\\{ l_{a} : 1.0, \\  l_{b} : 4.0, \\  l_{c} : 3.0, \\  l_{n} : 5.0, \\  q_{1} : 0.523598775598299\\right\\}$"
      ],
      "text/plain": [
       "{lₐ: 1.0, l_b: 4.0, l_c: 3.0, lₙ: 5.0, q₁: 0.5235987755982988}"
      ]
     },
     "execution_count": 11,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "import math  # provides pi as a float\n",
    "\n",
    "repl = {\n",
    "    la: 1.0,\n",
    "    lb: 4.0,\n",
    "    lc: 3.0,\n",
    "    ln: 5.0,\n",
    "    q1: 30.0/180.0*math.pi,  # 30 degrees in radians\n",
    "}\n",
    "repl"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "id": "47bb9200",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.727970Z",
     "iopub.status.busy": "2024-08-12T10:00:16.727829Z",
     "iopub.status.idle": "2024-08-12T10:00:16.740909Z",
     "shell.execute_reply": "2024-08-12T10:00:16.740521Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}4.0 \\cos{\\left(q_{2} + 0.523598775598299 \\right)} + 3.0 \\cos{\\left(q_{2} + q_{3} + 0.523598775598299 \\right)} - 4.13397459621556\\\\4.0 \\sin{\\left(q_{2} + 0.523598775598299 \\right)} + 3.0 \\sin{\\left(q_{2} + q_{3} + 0.523598775598299 \\right)} + 0.5\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡4.0⋅cos(q₂ + 0.523598775598299) + 3.0⋅cos(q₂ + q₃ + 0.523598775598299) - 4.13\n",
       "⎢                                                                             \n",
       "⎣      4.0⋅sin(q₂ + 0.523598775598299) + 3.0⋅sin(q₂ + q₃ + 0.523598775598299) \n",
       "\n",
       "397459621556⎤\n",
       "            ⎥\n",
       "+ 0.5       ⎦"
      ]
     },
     "execution_count": 12,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "fh.xreplace(repl)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "id": "33220558",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.742533Z",
     "iopub.status.busy": "2024-08-12T10:00:16.742392Z",
     "iopub.status.idle": "2024-08-12T10:00:16.804671Z",
     "shell.execute_reply": "2024-08-12T10:00:16.804249Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}-79.9561178980214\\\\108.613175851763\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡-79.9561178980214⎤\n",
       "⎢                 ⎥\n",
       "⎣108.613175851763 ⎦"
      ]
     },
     "execution_count": 13,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "q2_guess = -75.0/180.0*math.pi  # -75 degrees in radians\n",
    "q3_guess = 100.0/180.0*math.pi  # 100 degrees in radians\n",
    "\n",
    "sol = sm.nsolve(fh.xreplace(repl), (q2, q3), (q2_guess, q3_guess))\n",
    "sol/math.pi*180.0  # to degrees"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "id": "901f79bd",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.806507Z",
     "iopub.status.busy": "2024-08-12T10:00:16.806205Z",
     "iopub.status.idle": "2024-08-12T10:00:16.810759Z",
     "shell.execute_reply": "2024-08-12T10:00:16.810366Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left\\{ a : 1.0, \\  b : 4.0, \\  q_{2} : 4.62512251778497 - q_{1}\\right\\}$"
      ],
      "text/plain": [
       "{a: 1.0, b: 4.0, q₂: 4.62512251778497 - q₁}"
      ]
     },
     "execution_count": 14,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "repl = {\n",
    "    a: 1.0,\n",
    "    b: 4.0,\n",
    "    q2: 3.0*math.pi/2.0 - 5.0/180.0*math.pi - q1,\n",
    "}\n",
    "repl"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "id": "a35270f3",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.812352Z",
     "iopub.status.busy": "2024-08-12T10:00:16.812212Z",
     "iopub.status.idle": "2024-08-12T10:00:16.822694Z",
     "shell.execute_reply": "2024-08-12T10:00:16.822314Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}4.0 \\cos{\\left(q_{3} + 4.62512251778497 \\right)} + 4.0 \\cos{\\left(q_{1} \\right)} - 7.97431148549532\\\\4.0 \\sin{\\left(q_{3} + 4.62512251778497 \\right)} + 4.0 \\sin{\\left(q_{1} \\right)} + 0.00761060381650891\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡ 4.0⋅cos(q₃ + 4.62512251778497) + 4.0⋅cos(q₁) - 7.97431148549532  ⎤\n",
       "⎢                                                                  ⎥\n",
       "⎣4.0⋅sin(q₃ + 4.62512251778497) + 4.0⋅sin(q₁) + 0.00761060381650891⎦"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "fh_watts.xreplace(repl)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "id": "cee95312",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.824554Z",
     "iopub.status.busy": "2024-08-12T10:00:16.824181Z",
     "iopub.status.idle": "2024-08-12T10:00:16.856153Z",
     "shell.execute_reply": "2024-08-12T10:00:16.855666Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}4.53780194767253\\\\90.3528330377729\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡4.53780194767253⎤\n",
       "⎢                ⎥\n",
       "⎣90.3528330377729⎦"
      ]
     },
     "execution_count": 16,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "q1_guess = 10.0/180.0*math.pi\n",
    "q3_guess = 100.0/180.0*math.pi\n",
    "\n",
    "sol = sm.nsolve(fh_watts.xreplace(repl), (q1, q3), (q1_guess, q3_guess))\n",
    "sol/math.pi*180.0  # to degrees"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "id": "a91b8682",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.858184Z",
     "iopub.status.busy": "2024-08-12T10:00:16.857774Z",
     "iopub.status.idle": "2024-08-12T10:00:16.861661Z",
     "shell.execute_reply": "2024-08-12T10:00:16.861186Z"
    }
   },
   "outputs": [],
   "source": [
    "P1 = me.Point('P1')\n",
    "P2 = me.Point('P2')\n",
    "P3 = me.Point('P3')\n",
    "P4 = me.Point('P4')\n",
    "P2.set_pos(P1, la*A.x)\n",
    "P3.set_pos(P2, lb*B.x)\n",
    "P4.set_pos(P3, lc*C.x)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "id": "ec3accec",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.863559Z",
     "iopub.status.busy": "2024-08-12T10:00:16.863185Z",
     "iopub.status.idle": "2024-08-12T10:00:16.885445Z",
     "shell.execute_reply": "2024-08-12T10:00:16.884964Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{c} \\left(\\dot{q}_{1} + \\dot{q}_{2} + \\dot{q}_{3}\\right)\\hat{c}_y + l_{b} \\left(\\dot{q}_{1} + \\dot{q}_{2}\\right)\\hat{b}_y + l_{a} \\dot{q}_{1}\\hat{a}_y$"
      ],
      "text/plain": [
       "l_c⋅(q₁̇ + q₂̇ + q₃̇) c_y + l_b⋅(q₁̇ + q₂̇) b_y + lₐ⋅q₁̇ a_y"
      ]
     },
     "execution_count": 18,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "P1.set_vel(N, 0)\n",
    "P4.vel(N)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 19,
   "id": "197ace02",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:16.887305Z",
     "iopub.status.busy": "2024-08-12T10:00:16.887044Z",
     "iopub.status.idle": "2024-08-12T10:00:17.428346Z",
     "shell.execute_reply": "2024-08-12T10:00:17.427795Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle - l_{a} \\sin{\\left(q_{1} \\right)} \\dot{q}_{1} - l_{b} \\left(\\dot{q}_{1} + \\dot{q}_{2}\\right) \\sin{\\left(q_{1} + q_{2} \\right)} - l_{c} \\left(\\dot{q}_{1} + \\dot{q}_{2} + \\dot{q}_{3}\\right) \\sin{\\left(q_{1} + q_{2} + q_{3} \\right)}$"
      ],
      "text/plain": [
       "-lₐ⋅sin(q₁)⋅q₁̇ - l_b⋅(q₁̇ + q₂̇)⋅sin(q₁ + q₂) - l_c⋅(q₁̇ + q₂̇ + q₃̇)⋅sin(q₁ \n",
       "+ q₂ + q₃)"
      ]
     },
     "execution_count": 19,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "sm.trigsimp(P4.vel(N).dot(N.x))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 20,
   "id": "78daf94b",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:17.430638Z",
     "iopub.status.busy": "2024-08-12T10:00:17.430225Z",
     "iopub.status.idle": "2024-08-12T10:00:17.955778Z",
     "shell.execute_reply": "2024-08-12T10:00:17.955325Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{a} \\cos{\\left(q_{1} \\right)} \\dot{q}_{1} + l_{b} \\left(\\dot{q}_{1} + \\dot{q}_{2}\\right) \\cos{\\left(q_{1} + q_{2} \\right)} + l_{c} \\left(\\dot{q}_{1} + \\dot{q}_{2} + \\dot{q}_{3}\\right) \\cos{\\left(q_{1} + q_{2} + q_{3} \\right)}$"
      ],
      "text/plain": [
       "lₐ⋅cos(q₁)⋅q₁̇ + l_b⋅(q₁̇ + q₂̇)⋅cos(q₁ + q₂) + l_c⋅(q₁̇ + q₂̇ + q₃̇)⋅cos(q₁ +\n",
       " q₂ + q₃)"
      ]
     },
     "execution_count": 20,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "sm.trigsimp(P4.vel(N).dot(N.y))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 21,
   "id": "f6175e77",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:17.957769Z",
     "iopub.status.busy": "2024-08-12T10:00:17.957378Z",
     "iopub.status.idle": "2024-08-12T10:00:17.987340Z",
     "shell.execute_reply": "2024-08-12T10:00:17.986812Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}- l_{a} \\sin{\\left(q_{1} \\right)} \\dot{q}_{1} - l_{b} \\left(\\dot{q}_{1} + \\dot{q}_{2}\\right) \\sin{\\left(q_{1} + q_{2} \\right)} - l_{c} \\left(\\dot{q}_{1} + \\dot{q}_{2} + \\dot{q}_{3}\\right) \\sin{\\left(q_{1} + q_{2} + q_{3} \\right)}\\\\l_{a} \\cos{\\left(q_{1} \\right)} \\dot{q}_{1} + l_{b} \\left(\\dot{q}_{1} + \\dot{q}_{2}\\right) \\cos{\\left(q_{1} + q_{2} \\right)} + l_{c} \\left(\\dot{q}_{1} + \\dot{q}_{2} + \\dot{q}_{3}\\right) \\cos{\\left(q_{1} + q_{2} + q_{3} \\right)}\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡-lₐ⋅sin(q₁)⋅q₁̇ - l_b⋅(q₁̇ + q₂̇)⋅sin(q₁ + q₂) - l_c⋅(q₁̇ + q₂̇ + q₃̇)⋅sin(q₁\n",
       "⎢                                                                             \n",
       "⎣lₐ⋅cos(q₁)⋅q₁̇ + l_b⋅(q₁̇ + q₂̇)⋅cos(q₁ + q₂) + l_c⋅(q₁̇ + q₂̇ + q₃̇)⋅cos(q₁ \n",
       "\n",
       " + q₂ + q₃)⎤\n",
       "     ⎥\n",
       "+ q₂ + q₃) ⎦"
      ]
     },
     "execution_count": 21,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "t = me.dynamicsymbols._t\n",
    "fhd = fh.diff(t)\n",
    "fhd"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 22,
   "id": "01585ba3",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:17.989489Z",
     "iopub.status.busy": "2024-08-12T10:00:17.989008Z",
     "iopub.status.idle": "2024-08-12T10:00:17.994240Z",
     "shell.execute_reply": "2024-08-12T10:00:17.993755Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}\\dot{q}_{2}\\\\\\dot{q}_{3}\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡q₂̇⎤\n",
       "⎢  ⎥\n",
       "⎣q₃̇⎦"
      ]
     },
     "execution_count": 22,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "x = sm.Matrix([q2.diff(t), q3.diff(t)])\n",
    "x"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 23,
   "id": "350297e7",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:17.995808Z",
     "iopub.status.busy": "2024-08-12T10:00:17.995652Z",
     "iopub.status.idle": "2024-08-12T10:00:18.014578Z",
     "shell.execute_reply": "2024-08-12T10:00:18.014176Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}- l_{b} \\sin{\\left(q_{1} + q_{2} \\right)} - l_{c} \\sin{\\left(q_{1} + q_{2} + q_{3} \\right)} & - l_{c} \\sin{\\left(q_{1} + q_{2} + q_{3} \\right)}\\\\l_{b} \\cos{\\left(q_{1} + q_{2} \\right)} + l_{c} \\cos{\\left(q_{1} + q_{2} + q_{3} \\right)} & l_{c} \\cos{\\left(q_{1} + q_{2} + q_{3} \\right)}\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡-l_b⋅sin(q₁ + q₂) - l_c⋅sin(q₁ + q₂ + q₃)  -l_c⋅sin(q₁ + q₂ + q₃)⎤\n",
       "⎢                                                                 ⎥\n",
       "⎣l_b⋅cos(q₁ + q₂) + l_c⋅cos(q₁ + q₂ + q₃)   l_c⋅cos(q₁ + q₂ + q₃) ⎦"
      ]
     },
     "execution_count": 23,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "A = fhd.jacobian(x)\n",
    "A"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 24,
   "id": "d675cbf0",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:18.016163Z",
     "iopub.status.busy": "2024-08-12T10:00:18.015993Z",
     "iopub.status.idle": "2024-08-12T10:00:18.026926Z",
     "shell.execute_reply": "2024-08-12T10:00:18.026429Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}l_{a} \\sin{\\left(q_{1} \\right)} \\dot{q}_{1} + l_{b} \\sin{\\left(q_{1} + q_{2} \\right)} \\dot{q}_{1} + l_{c} \\sin{\\left(q_{1} + q_{2} + q_{3} \\right)} \\dot{q}_{1}\\\\- l_{a} \\cos{\\left(q_{1} \\right)} \\dot{q}_{1} - l_{b} \\cos{\\left(q_{1} + q_{2} \\right)} \\dot{q}_{1} - l_{c} \\cos{\\left(q_{1} + q_{2} + q_{3} \\right)} \\dot{q}_{1}\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡lₐ⋅sin(q₁)⋅q₁̇ + l_b⋅sin(q₁ + q₂)⋅q₁̇ + l_c⋅sin(q₁ + q₂ + q₃)⋅q₁̇ ⎤\n",
       "⎢                                                               ⎥\n",
       "⎣-lₐ⋅cos(q₁)⋅q₁̇ - l_b⋅cos(q₁ + q₂)⋅q₁̇ - l_c⋅cos(q₁ + q₂ + q₃)⋅q₁̇⎦"
      ]
     },
     "execution_count": 24,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "b = -fhd.xreplace({q2.diff(t): 0, q3.diff(t): 0})\n",
    "b"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 25,
   "id": "7286775c",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:18.028621Z",
     "iopub.status.busy": "2024-08-12T10:00:18.028467Z",
     "iopub.status.idle": "2024-08-12T10:00:19.468673Z",
     "shell.execute_reply": "2024-08-12T10:00:19.468234Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left[\\begin{matrix}- \\frac{\\left(\\frac{l_{a} \\sin{\\left(q_{2} \\right)}}{\\tan{\\left(q_{3} \\right)}} + l_{a} \\cos{\\left(q_{2} \\right)} + l_{b}\\right) \\dot{q}_{1}}{l_{b}}\\\\\\frac{l_{a} \\left(l_{b} \\sin{\\left(q_{2} \\right)} + l_{c} \\sin{\\left(q_{2} + q_{3} \\right)}\\right) \\dot{q}_{1}}{l_{b} l_{c} \\sin{\\left(q_{3} \\right)}}\\end{matrix}\\right]$"
      ],
      "text/plain": [
       "⎡  ⎛lₐ⋅sin(q₂)                   ⎞     ⎤\n",
       "⎢ -⎜────────── + lₐ⋅cos(q₂) + l_b⎟⋅q₁̇  ⎥\n",
       "⎢  ⎝ tan(q₃)                     ⎠     ⎥\n",
       "⎢ ──────────────────────────────────── ⎥\n",
       "⎢                 l_b                  ⎥\n",
       "⎢                                      ⎥\n",
       "⎢lₐ⋅(l_b⋅sin(q₂) + l_c⋅sin(q₂ + q₃))⋅q₁̇⎥\n",
       "⎢──────────────────────────────────────⎥\n",
       "⎣           l_b⋅l_c⋅sin(q₃)            ⎦"
      ]
     },
     "execution_count": 25,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "x_sol = sm.simplify(A.LUsolve(b))\n",
    "x_sol"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 26,
   "id": "f450e3f9",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:19.470427Z",
     "iopub.status.busy": "2024-08-12T10:00:19.470265Z",
     "iopub.status.idle": "2024-08-12T10:00:19.478148Z",
     "shell.execute_reply": "2024-08-12T10:00:19.477782Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left\\{q_{1}, q_{2}, q_{3}, \\dot{q}_{1}, \\dot{q}_{2}, \\dot{q}_{3}\\right\\}$"
      ],
      "text/plain": [
       "{q₁, q₂, q₃, q₁̇, q₂̇, q₃̇}"
      ]
     },
     "execution_count": 26,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "P4.vel(N).free_dynamicsymbols(N)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 27,
   "id": "84362628",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:19.480007Z",
     "iopub.status.busy": "2024-08-12T10:00:19.479702Z",
     "iopub.status.idle": "2024-08-12T10:00:19.488112Z",
     "shell.execute_reply": "2024-08-12T10:00:19.487699Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left\\{ \\dot{q}_{2} : - \\frac{\\left(\\frac{l_{a} \\sin{\\left(q_{2} \\right)}}{\\tan{\\left(q_{3} \\right)}} + l_{a} \\cos{\\left(q_{2} \\right)} + l_{b}\\right) \\dot{q}_{1}}{l_{b}}, \\  \\dot{q}_{3} : \\frac{l_{a} \\left(l_{b} \\sin{\\left(q_{2} \\right)} + l_{c} \\sin{\\left(q_{2} + q_{3} \\right)}\\right) \\dot{q}_{1}}{l_{b} l_{c} \\sin{\\left(q_{3} \\right)}}\\right\\}$"
      ],
      "text/plain": [
       "⎧     ⎛lₐ⋅sin(q₂)                   ⎞                                         \n",
       "⎪    -⎜────────── + lₐ⋅cos(q₂) + l_b⎟⋅q₁̇                                     \n",
       "⎨     ⎝ tan(q₃)                     ⎠          lₐ⋅(l_b⋅sin(q₂) + l_c⋅sin(q₂ + \n",
       "⎪q₂̇: ────────────────────────────────────, q₃̇: ─────────────────────────────\n",
       "⎩                    l_b                                  l_b⋅l_c⋅sin(q₃)     \n",
       "\n",
       "       ⎫\n",
       "        ⎪\n",
       "q₃))⋅q₁̇⎬\n",
       "─────────⎪\n",
       "       ⎭"
      ]
     },
     "execution_count": 27,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "qd_dep_repl = {\n",
    "  q2.diff(t): x_sol[0, 0],\n",
    "  q3.diff(t): x_sol[1, 0],\n",
    "}\n",
    "qd_dep_repl"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 28,
   "id": "eed3ea6c",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:19.489912Z",
     "iopub.status.busy": "2024-08-12T10:00:19.489582Z",
     "iopub.status.idle": "2024-08-12T10:00:19.501890Z",
     "shell.execute_reply": "2024-08-12T10:00:19.501358Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle l_{c} \\left(\\frac{l_{a} \\left(l_{b} \\sin{\\left(q_{2} \\right)} + l_{c} \\sin{\\left(q_{2} + q_{3} \\right)}\\right) \\dot{q}_{1}}{l_{b} l_{c} \\sin{\\left(q_{3} \\right)}} + \\dot{q}_{1} - \\frac{\\left(\\frac{l_{a} \\sin{\\left(q_{2} \\right)}}{\\tan{\\left(q_{3} \\right)}} + l_{a} \\cos{\\left(q_{2} \\right)} + l_{b}\\right) \\dot{q}_{1}}{l_{b}}\\right)\\hat{c}_y + l_{b} \\left(\\dot{q}_{1} - \\frac{\\left(\\frac{l_{a} \\sin{\\left(q_{2} \\right)}}{\\tan{\\left(q_{3} \\right)}} + l_{a} \\cos{\\left(q_{2} \\right)} + l_{b}\\right) \\dot{q}_{1}}{l_{b}}\\right)\\hat{b}_y + l_{a} \\dot{q}_{1}\\hat{a}_y$"
      ],
      "text/plain": [
       "    ⎛                                              ⎛lₐ⋅sin(q₂)\n",
       "    ⎜                                              ⎜────────── + lₐ⋅cos(q₂) +\n",
       "    ⎜lₐ⋅(l_b⋅sin(q₂) + l_c⋅sin(q₂ + q₃))⋅q₁̇        ⎝ tan(q₃)\n",
       "l_c⋅⎜────────────────────────────────────── + q₁̇ - ──────────────────────────\n",
       "    ⎝           l_b⋅l_c⋅sin(q₃)                                   l_b\n",
       "\n",
       "   ⎞   ⎞           ⎛     ⎛lₐ⋅sin(q₂)                   ⎞   ⎞\n",
       "l_b⎟⋅q₁̇⎟           ⎜     ⎜────────── + lₐ⋅cos(q₂) + l_b⎟⋅q₁̇⎟\n",
       "    ⎠   ⎟           ⎜     ⎝ tan(q₃)                     ⎠   ⎟\n",
       "────────⎟ c_y + l_b⋅⎜q₁̇ - ──────────────────────────────────⎟ b_y + lₐ⋅q₁̇ a_\n",
       "       ⎠           ⎝                    l_b                ⎠"
      ]
     },
     "execution_count": 28,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "P4.vel(N).xreplace(qd_dep_repl)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 29,
   "id": "1daf4983",
   "metadata": {
    "execution": {
     "iopub.execute_input": "2024-08-12T10:00:19.503565Z",
     "iopub.status.busy": "2024-08-12T10:00:19.503421Z",
     "iopub.status.idle": "2024-08-12T10:00:19.513451Z",
     "shell.execute_reply": "2024-08-12T10:00:19.512926Z"
    }
   },
   "outputs": [
    {
     "data": {
      "text/latex": [
       "$\\displaystyle \\left\\{q_{1}, q_{2}, q_{3}, \\dot{q}_{1}\\right\\}$"
      ],
      "text/plain": [
       "{q₁, q₂, q₃, q₁̇}"
      ]
     },
     "execution_count": 29,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "P4.vel(N).xreplace(qd_dep_repl).free_dynamicsymbols(N)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.11.9"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}